@@ -123,6 +123,20 @@ impl CostModel {
         transaction.signatures.len() as u64 * SIGNATURE_COST
     }
 
+    pub fn find_instruction_cost(&self, program_key: &Pubkey) -> u64 {
+        match self.instruction_execution_cost_table.get_cost(program_key) {
+            Some(cost) => *cost,
+            None => {
+                let default_value = self.instruction_execution_cost_table.get_mode();
+                debug!(
+                    "Program key {:?} does not have assigned cost, using mode {}",
+                    program_key, default_value
+                );
+                default_value
+            }
+        }
+    }
+
     fn get_write_lock_cost(
         &self,
         tx_cost: &mut TransactionCost,
@@ -164,20 +178,6 @@ impl CostModel {
         }
         cost
     }
-
-    fn find_instruction_cost(&self, program_key: &Pubkey) -> u64 {
-        match self.instruction_execution_cost_table.get_cost(program_key) {
-            Some(cost) => *cost,
-            None => {
-                let default_value = self.instruction_execution_cost_table.get_mode();
-                debug!(
-                    "Program key {:?} does not have assigned cost, using mode {}",
-                    program_key, default_value
-                );
-                default_value
-            }
-        }
-    }
 }
 
 #[cfg(test)]
