@@ -15,7 +15,6 @@ pub mod bank_forks;
 pub mod bank_utils;
 pub mod block_cost_limits;
 pub mod blockhash_queue;
-pub mod bloom;
 pub mod builtins;
 pub mod commitment;
 pub mod contains;
@@ -26,7 +25,7 @@ pub mod execute_cost_table;
 pub mod genesis_utils;
 pub mod hardened_unpack;
 pub mod hashed_transaction;
-pub mod inline_spl_token_v2_0;
+pub mod inline_spl_token;
 pub mod instruction_recorder;
 pub mod loader_utils;
 pub mod log_collector;
